// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import ConfigurationServices
import ErrorData
import GMAppData

extension SettingsDataAccessView {
    final class ViewModel: ObservableObject {
        @Published var didAppear: Bool = false
        #warning("Implement 'shouldSendData' action!")
        @Published var shouldSendData: Bool = false
        @Published var isReceiveDataAfterCourseEnabled: Bool = false
        @Published var isLoadingReceiveDataAfterCourse: Bool = false
        @Published var error: Error?

        @DI private var configurationService: ConfigurationService
        @DI private var maternityDataEnvironment: MaternityDataEnvironmentStore

        private var cancelBag = CancelBag()

        func fetchData() {
            guard let maternityId = maternityDataEnvironment.selectedMaternity?.episodeOfCareIdentifier else {
                error = ServiceError.unexpectedError(nil)
                return
            }

            isLoadingReceiveDataAfterCourse = true
            configurationService
                .fetchPDFStatus(maternityId: maternityId)
                .receiveOnMain()
                .sink(receiveCompletion: { [weak self] (completion) in
                    switch completion {
                    case .failure(let error):
                        self?.error = error
                    case .finished: break
                    }
                    self?.isLoadingReceiveDataAfterCourse = false
                }, receiveValue: { [weak self] (pdf) in
                    self?.isReceiveDataAfterCourseEnabled = pdf.wantsPDF ?? false
                })
                .store(in: &cancelBag)
        }

        func updatePDFStatus() {
            guard let maternityId = maternityDataEnvironment.selectedMaternity?.episodeOfCareIdentifier else {
                error = ServiceError.unexpectedError(nil)
                return
            }
            isLoadingReceiveDataAfterCourse = true
            configurationService
                .updatePDFStatus(maternityId: maternityId, wantsPDF: isReceiveDataAfterCourseEnabled)
                .receiveOnMain()
                .sink(receiveCompletion: { [weak self] (completion) in
                    switch completion {
                    case .failure(let error):
                        self?.error = error
                    case .finished: break
                    }
                    self?.isLoadingReceiveDataAfterCourse = false
                }, receiveValue: { [weak self] (pdf) in
                    self?.isReceiveDataAfterCourseEnabled = pdf.wantsPDF ?? false
                })
                .store(in: &cancelBag)
        }
    }
}
